local rev = tonumber(("$Revision: 78816 $"):match("(%d+)")) or 0
if rev > GuildCraft.version then GuildCraft.version = rev end

local D, newTable, delTable, clearTable
do
	function D(msg)
		if (not GuildCraft ) or (not GuildCraft.db.global.debug.dbo) then return end
		DEFAULT_CHAT_FRAME:AddMessage("|cff33FF66GC2-DBO:|r "..msg)
	end

-- Table Pool for recycling tables
	local tablePool = {}
	setmetatable(tablePool, {__mode = "kv"});	-- Weak table

	function newTable()
		local t = next(tablePool) or {};
		tablePool[t] = nil;
		return t;
	end

	function delTable(t)
		if (type(t) == "table") then
			for k, v in pairs(t) do
				if (type(v) == "table") then
					delTable(v);	-- child tables get put into the pool
				end
				t[k] = nil;
			end
			setmetatable(t, nil);
			t[true] = true;
			t[true] = nil;
			tablePool[t] = true;
		end
		return nil;
	end
	
	function clearTable(t)
		for k,v in pairs(t) do
			if type(v) == 'table' then
				t[k] = delTable(t[k])
			else
				t[k] = nil
			end
		end
		return t
	end
end


--[[  HOUSEKEEPING  ]]--
local AddonParent = LibStub("AceAddon-3.0"):GetAddon("GuildCraft")
local module = AddonParent:NewModule("DBO") --Get Toys
module.rev = rev
local self_player = UnitName("player")
local dbRemoteInfo, dbCache, CraftSelect, CommsIO
local L = LibStub("AceLocale-3.0"):GetLocale("GuildCraft")
local BTS = LibStub("GuildCraft_BabbleTradeSkill_Mini")
local unitGuild

function module:OnEnable()
	unitGuild = GetGuildInfo("player")
	--
	CommsIO = AddonParent:GetModule("CommsIO")
	CraftSelect = AddonParent:GetModule("CraftSelect")
	dbRemoteInfo = AddonParent.db.factionrealm.RemoteInfo
	dbCache = AddonParent.Cache
	--
	CommsIO:SoftEmbed(self)
	self:RegisterCommsCall("VersionUpdateRequest")
	self:RegisterCommsCall("HostedVersion")
	self:RegisterCommsCall("GiveHostedVerion")
	self:RegisterCommsCall("UpdateTradeInfo")
	--
	self:RegisterCommsCall("AddonVersion")
	self:RegisterCommsCall("SendAddonVersion")
	--
	self:RegisterCommsCall("CheckOfflineVersions")
	self:RegisterCommsCall("GiveOfflineVersion")
	self:RegisterCommsCall("UpdateOfflineData")
	--
	--self:RegisterCommsCall("")
	--
	self:RefreshCache()		--build our cache table
	self:RequestCurrentVersion()	--update people online
	self:RequestAddonVersion()	--Get online player data
	--self:StarOfflineCheck()	--Get Offline Player Data
	D("DBO LOADED")
end

function module:GetLocalVersion(sender)
	--D("GetLocalVersion")
	return AddonParent.db.char.dbVersion or -1
end

function module:GetRemoteVersion(sender)
	if (dbRemoteInfo[sender]) and (dbRemoteInfo[sender].dbVersion) then
		--D("GetRemoteVersion: "..sender.." ver "..dbRemoteInfo[sender].dbVersion)
		return dbRemoteInfo[sender].dbVersion
	end
end

	--Dialouge
function module:RequestCurrentVersion()
	D("ReqCurrVersion - Start()")
	if self:IsLocked() then 
		return 
	end
	self:SendToGuild("VersionUpdateRequest")
end

function module:VersionUpdateRequest(event, sender)
	if self:IsLocked() then
		return 
	end
	if tonumber(self:GetLocalVersion()) == -1 then return end
	D("VerUpdReq Called from "..sender.." sending 'HostedVer' ")
	self:SendToGuild( "HostedVersion", self:GetLocalVersion()  )
end

function module:HostedVersion(event, sender, version)
	if self:IsLocked() then 
		return 
	end
	version = tonumber(version)
	local rVer = tonumber(self:GetRemoteVersion(sender) or "-1")
	if (rVer and rVer < version) then
		D("HostedVer Called from "..sender.. "sending 'GiveHosted' as they have update")
		self:SendWhisper( sender, "GiveHostedVerion" )
	end
end

function module:GiveHostedVerion(event, sender, version , tbl)
	D("GiveHostedVer called from "..sender.." blindly sending share table")
	self:SendWhisper( sender, "UpdateTradeInfo", self:GetLocalVersion(), CraftSelect:GetShareTable() )
end

function module:UpdateTradeInfo(event, sender, version , tbl)
	D("Update Trade Info()")
	version = tonumber(version)
	local rVer = tonumber(self:GetRemoteVersion(sender) or "-1")
	D("UTI incVer: "..version.." lVer: "..rVer.." "..(rVer < version and "Inc>Local" or "local > Inc"))
	if (rVer and rVer < version) then
		if self:UpdateRemoteSender(sender, version, tbl) then
			AddonParent:Print(L["Updated %s's tradeskills. Version: %s"], sender, version)
		end
	end
end

function module:RequestAddonVersion()
	self:SendToGuild("SendAddonVersion")
end
function module:SendAddonVersion()
	D("Sending addon version to guild")
	self:SendToGuild("AddonVersion", GuildCraft.version)
end

function module:AddonVersion(event, sender, version)
	D("Recived another addon version from "..sender)
	AddonParent.db.global.playerVersions[sender] = version
end

function module:PublishToGuild()
	D("Publishing to guild()")
	if self:IsLocked() then
		return 
	end
	self:SendToGuild( "UpdateTradeInfo", self:GetLocalVersion(), CraftSelect:GetShareTable() )
end

function module:UpdateRemoteSender(sender, version, tbl)
	D("UpdateRemoteSender()")
	if AddonParent.db.factionrealm.guildSettings[unitGuild].blacklisted[sender] then return end
	version = tonumber(version)
	local rVer = tonumber(self:GetRemoteVersion(sender) or "-1")
	if (rVer and rVer < version) then
		D("Updating Remote sender for real here")
		dbRemoteInfo[sender] = tbl
		dbRemoteInfo[sender].dbVersion = version
		self:RefreshCache(sender, tbl)
		return true
	end
	return false
end

----Cache
local guildTrades = {}
--Function designed to finalize updates for the cache table in real time and on boot

function module:RefreshCache(player, specificTable)
	D("Refreshing Cache "..(player and "For player "..player or "all") )
	if specificTable then	--Means that we have individual update
		for trade, data in pairs(specificTable) do	--Dive into provided table
			if (type(data) == "table") and BTS[trade] then	--skip over the dbVersion that we just added to it
				--D("trade for player "..trade)
				dbCache[trade] = dbCache[trade] or newTable()	--Init Cache table
				local tradeTable = dbCache[trade]	--Get and save cache table
				for item, _ in pairs(data) do	--Pair over data
					tradeTable[item] = tradeTable[item] or ""	--add item to table or get existing item
					--D( (tradeTable[item] == nil and "Error on tradeTable[item]" or tradeTable[item]).." type: "..type(tradeTable[item]))
					if not tradeTable[item]:match(player) then	--check if we all ready know this
						if tradeTable[item] == "" then --if it's new than just add play'ers name to it
							tradeTable[item] = player
						else
							tradeTable[item] = tradeTable[item]..", "..player	--else add to end; consider sorting players alphabeticly
						end
						--D("add: "..player..", as make for: "..item..", trade: "..trade)
					end
				end
				if next(tradeTable) then	--if something is there, then add to guild craftables
					guildTrades[trade] = true
				else
					dbCache[trade] = delTable(dbCache[trade])	--else remove table
				end
			elseif type(data) == "table" and not BTS[trade] then
				AddonParent:Print(L["Invalid Data from %s, %s is invalid"], player, trade)	--tradeskill not in BTS so end
			end
		end
	else		---Cache Table init
		dbCache = clearTable(dbCache)
		--D( (dbCache == AddonParent.Cache and "CacheTable true" or "Cache Table False"))
		guildTrades = clearTable(guildTrades)
		for player, tradeData in pairs(dbRemoteInfo) do		--Dive into Master table
			D("Refreshing Player "..player)
			for trade, data in pairs(tradeData) do		--pairs though it...
				if (type(data) == "table" and BTS[trade]) then		--make sure we pair into a table and that it's valid
					--D("Refreshing "..trade.." for "..player)
					dbCache[trade] = dbCache[trade] or newTable()		-- Set same or make new table on first walk, easy upgrading
					local tradeTable = dbCache[trade]
					for item, _ in pairs(data) do	--Pair though data to create...
						tradeTable[item] = tradeTable[item] or ""
						if not tradeTable[item]:match(player) then
							if tradeTable[item] == "" then 
								tradeTable[item] = player
							else
								tradeTable[item] = tradeTable[item]..", "..player
							end
							--D("add: "..player..", as make for: "..item..", trade: "..trade)
						end
					end
					if next(tradeTable) then
						guildTrades[trade] = true
					else
						dbCache[trade] = delTable(dbCache[trade])
					end
				elseif type(data) == "table" and not BTS[trade] then
					AddonParent:Print(L["Invalid Data from %s, %s is invalid"], player, trade)	--Invalid tradeskill...
				end
			end
		end
	end
end

function module:GetGuildTradesList()
	return guildTrades
end
function module:GetGuildTrade(trade)
	return dbCache[trade] and dbCache[trade] or nil
end

function module:GetPrittyMakers(trade, item)
	return ((dbCache[trade] and dbCache[trade][item]) and dbCache[trade][item])
end

------Guild Management Functions

function module:SetDBOLocked(status, who)
	D("SetLocked() "..(status and "true" or "false"))
	if not status then	--if the DBO isn't locked, tell everyone what version we have, incase we updated recently while it was locked...
		self:VersionUpdateRequest()
	else
		AddonParent:Print(L["Database Locked by: %s"], who)
	end
end

function module:PurgeBlacklisted(...)
	D("Purge Blacklisted: "..(string.join(", ", ...) ) )
	for i=1, select("#", ...) do
		local player = select(i,...)
		if dbRemoteInfo[player] then
			dbRemoteInfo[player] = nil
			--D(player.." has been blacklisted by the Officers from Guild Craft")
		end
	end
end

function module:IsLocked()
	unitGuild = GetGuildInfo("player")
		if unitGuild then
			local tbl = AddonParent.db.factionrealm.guildSettings[unitGuild]
			return tbl and tbl.locked or false
		else
			return false
		end
end

-------------Offline Synch
module.dbV = {}
local dbVersion = setmetatable(module.dbV,{
	__index = function(t,k,v)
		rawset(t, k, dbRemoteInfo[k] and dbRemoteInfo[k].dbVersion or nil)
		return dbRemoteInfo[k] and dbRemoteInfo[k].dbVersion or nil
	end,
})

function GetOfflinePlayerString()
	local str = ""
	--D( (next(AddonParent.offline) or "ERROR").." table has data?")
	for k, _ in pairs(AddonParent.offline) do 		--we gathered offline data eairlyer on core's OnEnable / thing...
		--D(k.." is offline")
		if dbRemoteInfo[k] then
			--D(k.." has data")
			if dbRemoteInfo[k].dbVersion then		--if the person offline is in our database
				--D(k.." has data and is offline")
				str = str..k..":"..dbRemoteInfo[k].dbVersion..";"	--get their version and addo to string
			end
		end
	end
	return str ~= "" and str or false	--if we're noobs and have nothing then say so, else return string.
end

local function GetData(player)
	local data = dbRemoteInfo[player]
	if data then
		return data, dbVersion[player]
	else
		return nil
	end
end


function module:StarOfflineCheck()
	D("StartOfflineChecking")
	local str = GetOfflinePlayerString()
	if str then
		self:SendToGuild("CheckOfflineVersions", str )
		D("Offline string: "..str)
	else
		D("No one known to be Offline")
	end
end

function module:CheckOfflineVersions(event, sender, str, ...)
	--D("Start "..event.." "..sender)
	if not str then
		--D("no string")
		return 
	end
	for player, version in string.gmatch(str, "(%w+):(%d+);") do		--match though string for player/version pairs
		version = tonumber(version)
		--D("For "..player.." and "..version.." do")
		local ver = dbVersion[player]	--easy reference rather than repetitive lookup
		--D("local v:"..ver)
		if ver then		--Ok, Player in our database
			--D("if Ver & true")
			if version > ver then	--if incoming version is better...
				--D("forign version better, asking for it")
				self:SendWhisper(sender, "GiveOfflineVersion", player)		--Ask other player for the offline data
			elseif ver > version then	--if our version is better...
				--D("local ver better, sending ours")
				--self:SendToGuild()		-- tell other player we got better data
				self:StarOfflineCheck()
			end
		end
		if not ver then			--Offline Player not in database
			--D("no version for data, asking for it")
			self:SendWhisper(sender, "GiveOfflineVersion", player)	--ask for offline data
		end
		if version == ver then
			--D("All Same Data")
		end
	end
end

function module:GiveOfflineVersion(event, sender, player)
	local data, version = GetData(player)
	if data then
		self:SendWhisper(sender, "UpdateOfflineData", player, version, data )
	end
end

function module:UpdateOfflineData(event, sender, player, version, data)
	self:UpdateRemoteSender(player, version, data)
end
