--ChoicesUI
local rev = tonumber(("$Revision: 78723 $"):match("(%d+)")) or 0
if rev > GuildCraft.version then GuildCraft.version = rev end

--GenericDebugFunc
local function D(msg)
	if (not GuildCraft ) or (not GuildCraft.db.global.debug.fubar) then return end
	ChatFrame1:AddMessage("|cff33FF66GC2-FB3:|r "..msg)
end

local AddonParent = LibStub("AceAddon-3.0"):GetAddon("GuildCraft")
local module = AddonParent:NewModule("FubarPlugin", "LibFuBarPlugin-Mod-3.0") --Get Toys
module.rev = rev
local L = LibStub("AceLocale-3.0"):GetLocale("GuildCraft")
local ReturnFubarOptions
local fubarOptions = {
	["tooltipType"] = "GameTooltip",
	["hasNoColor"] = true,
	["cannotDetachTooltip"] = true,
	["hideWithoutStandby"] = true,
	["iconPath"] =  "Interface\\Icons\\Ability_Repair",
	["defaultPosition"] = "MINIMAP",
	["defaultMinimapPosition"] = 35
	--[""] = "",
}
local FubarDefaultOptions = {
	profile = {
		["showText"] = true,
		["showIcon"] = true,
		["attached"] = false,
		["HideMinimapButton"] = false,
		["showExtTT"] = true,
	},
}

function module:OnEnable()
	-- Create the FuBarPlugin bits.
	for k,v in pairs(fubarOptions) do
		self:SetFuBarOption(k,v)
	end
	self:SetFuBarIcon("Interface\\Icons\\Ability_Repair")
	module.db = AddonParent.db:RegisterNamespace("FuBarOptions", FubarDefaultOptions)
	profile = self.db.profile
	AddonParent.optionsTable.args["FubarOptions"] = ReturnFubarOptions(module, module.db.profile)
	
	AddonParent.optionsTable.args["FubarOptions"].args["showExtTT"] = {
	name = L["Show Version for Realm"], type = "toggle", 
	get = function() return module.db.profile.showRealmDBV end,
	set = function(info, status) module.db.profile.showRealmDBV = status end,
	order = 15,}
	D("Fubar Module Loaded")
end

local function getReadableDBV(ver)
	local _, _, year, month, day, hr, minu, sec = string.find(ver, "(%d%d%d%d)(%d%d)(%d%d)(%d%d)(%d%d)(%d%d)")
	local str = string.format( "%d / %d / %d", year, month, day)
	return str or " "
end

function module:OnUpdateFuBarTooltip()	
	D("OnUpdateFubarTooltip")
	GameTooltip:AddLine(L["Guild Craft"])
	GameTooltip:AddLine("|cffffff00" .. L["Click|r to toggle the Browser"])
	GameTooltip:AddLine("|cffffff00" .. L["Right-click|r to open the options menu"])
	if self.db.profile.showRealmDBV then
		GameTooltip:AddLine(" ")
		local RI = AddonParent.db.factionrealm.RemoteInfo
		local aV = AddonParent.db.global.playerVersions
		local key = {}
		for k,v in pairs( RI ) do
			tinsert(key, k)
		end
		table.sort(key)
		for i,v in ipairs(key) do
			--D("Add "..v.." distro: "..(aV[v] or "").." dbV: "..(RI[v].dbVersion or ""))
			GameTooltip:AddDoubleLine(v..( aV[v] and (" - "..aV[v]) or "" ), getReadableDBV(RI[v].dbVersion))
		end
		GameTooltip:AddDoubleLine(" ", "YYYY/MM/DD")
	end
end

function module:OnFuBarClick(button)
	D("OnClick")
	if AddonParent:GetModule("BrowserUI", true) then
		AddonParent:GetModule("BrowserUI"):ToggleBrowser()
	end
end
function module:OnFuBarUpdateText()
	self:SetFuBarText(L["Guild Craft"])
end

module.OpenMenu = function(frame)
	D("Open Menu")
	if LibStub("AceConfigDialog-3.0").OpenFrames["Guild Craft"] then
		LibStub("AceConfigDialog-3.0"):Close("Guild Craft")
	else
		LibStub("AceConfigDialog-3.0"):Open("Guild Craft")
	end
end
module.title = L["Guild Craft"]

function ReturnFubarOptions( object, db )
	return {
	type = "group",
	name = L["FuBar Options"],
	desc = L["FuBar Options"],
	order = -10,
	args = {
		attachMinimap = {
			type = "toggle",
			name = L["Attach to minimap"],
			desc = L["Attach to minimap"],
			get = function(info)
				return object:IsFuBarMinimapAttached()
			end,
			set = function(info, v)
				if db.HideMinimapButton then return end
				object:ToggleFuBarMinimapAttached()
				db.attached = object:IsFuBarMinimapAttached()
			end,
			disabled = function(info) return db.HideMinimapButton end
		},
		HidePlugin = {
			type = "toggle",
			name = L["Hide Plugin"],
			desc = L["Hide minimap/FuBar icon"],
			get = function(info) return db.HideMinimapButton end,
			set = function(info, v)
				db.HideMinimapButton = v
				db.isHidden = v
				if not v then
					object:Show()
				else
					object:Hide()
				end
			end
		},
		showIcon = {
			type = "toggle",
			name = L["Show icon"],
			desc = L["Show icon"],
			get = function(info) return db.showIcon end,
			set = function(info, v) 
				db.showIcon = object:IsFuBarIconShown()
				module:ToggleFuBarIconShown() end,
			disabled = function(info) return db.isHidden end
		},
		showText = {
			type = "toggle",
			name = L["Show text"],
			desc = L["Show text"],
			get = function(info) return db.showText end,
			set = function(info, v)
				db.showText = object:IsFuBarTextShown()
				object:ToggleFuBarTextShown()
			end,
			disabled = function(info) 
				if db.isHidden then
					return true
				end
			end,
		},
		position = {
			type = "select",
			name = L["Position"],
			desc = L["Position"],
			values = {LEFT = L["Left"], CENTER = L["Center"], RIGHT = L["Right"]},
			get = function() return object:GetPanel() and object:GetPanel():GetPluginSide(object) end,
			set = function(info, val)
				if object:GetPanel() and object:GetPanel().SetPluginSide then
					object:GetPanel():SetPluginSide(object, val)
				end
			end,
			disabled = function(info) return object:IsFuBarMinimapAttached() or db.isHidden end
		}
	}
}
end

