local rev = tonumber(("$Revision: 78723 $"):match("(%d+)")) or 0
if rev > GuildCraft.version then GuildCraft.version = rev end

-------------
local function D(msg)
	if (not GuildCraft ) or (not GuildCraft.db.global.debug.browser) then return end
	ChatFrame1:AddMessage("|cff33FF66GC-BUI:|r "..msg)
end

local newtable, deltable
do
	-- Table Pool for recycling tables
	local tablePool = {};
	setmetatable(tablePool, {__mode = "kv"});	-- Weak table

	-- Get a new table
	function newtable()
		local t = next(tablePool) or {};
		tablePool[t] = nil;
		return t;
	end

	-- Delete table and add to table pool
	function deltable(t)
		if (type(t) == "table") then
			for k, v in pairs(t) do
				if (type(v) == "table") then
					deltable(v);	-- child tables get put into the pool
				end
				t[k] = nil;
			end
			setmetatable(t, nil);
			t[true] = true;
			t[true] = nil;
			tablePool[t] = true;
		end
		return nil;
	end
end
-------------------------------------------

local AddonParent = LibStub("AceAddon-3.0"):GetAddon("GuildCraft")
local Module = AddonParent:NewModule("BrowserUI","AceConsole-3.0")
Module.rev = rev
local AceGUI = LibStub("AceGUI-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("GuildCraft")
local BTS = LibStub("GuildCraft_BabbleTradeSkill_Mini")
local LibGratuity = LibStub("LibGratuity-3.0")

--Global vars that need to always be available to file.
local reqLevlCapture = ITEM_MIN_LEVEL:gsub("%%%a", "(%1+)")
local TreeView_Lists = {}
local DropDown_List = {}
local currentTradeSkill = nil
local currentSortMethod = nil
local sortByList = {}
local CacheDB, DBO, playerLvl

-- module usual function
function Module:OnInitialize()
	CacheDB = AddonParent.Cache
	DBO = AddonParent:GetModule("DBO")
end
function Module:OnEnable()
	self:RegisterChatCommand("gcb", self.ToggleBrowser )
	currentSortMethod = AddonParent.db.profile.displaySortMethod
	self:CreateBrowser()
	GuildCraftTT:SetBackdrop(nil)
	--self:ScheduleTimer("InitScan", 60)
	playerLvl = UnitLevel("player")
	D("Browser UI Loaded")
end

function Module:OnDisable()
	AddonParent.db.profile.displaySortMethod = currentSortMethod
end

--Module Utility Functions
local function getcolorcode(lvl)
	if lvl == -1 then
		return "|c00FFD000"
	end
	local tbl = GetDifficultyColor(lvl)
	return string.format("|c00%02X%02X%02X", tbl["r"] * 255, tbl["g"] * 255, tbl["b"] * 255) or ""
end

local redCC = "|c00FF1919"
local ecc = "|r"

local sortMethods = {
	Name = function(a,b)
		--D("sort, "..a.text.." & "..b.text)
		--if a.name and b.name then
			return a.text < b.text
		--end
		--return a.text < b.text	--:gsub("|c00%x+(.+)|r", "%1")
	end,
	ReqLvl = function(a,b)
		if (a.lvl and b.lvl) then
			return a.lvl > b.lvl
		end
		LibGratuity:SetHyperlink("enchant:"..a.value)
		local a_reqLvl = tonumber(select(3, LibGratuity:Find(reqLevlCapture)) or -1)
			--D(a_reqLvl.." and CC = "..getcolorcode(a_reqLvl))
			-- if not a.text:find("|c00") and getcolorcode(a_reqLvl) then
				-- a.text = getcolorcode(a_reqLvl)..a.text..ecc
			-- end
		LibGratuity:SetHyperlink("enchant:"..b.value)
		local b_reqLvl = tonumber(select(3, LibGratuity:Find(reqLevlCapture)) or -1)
			--D(b_reqLvl.." and CC = "..getcolorcode(b_reqLvl))
			-- if not a.text:find("|c00") and getcolorcode(b_reqLvl) then
				-- b.text = getcolorcode(b_reqLvl)..b.text..ecc
			-- end
		if a_reqLvl == b_reqLvl then
			return a.text < b.text
		end
		a.lvl = a_reqLvl
		b.lvl = b_reqLvl
		return a.lvl > b.lvl
	end,
	-- ItemLvl = function(a,b)
		-- local ilvl_A = (select(4,GetItemInfo(a.id)))
		-- local ilvl_B = (select(4,GetItemInfo(b.id)))
		-- return ilvl_A > ilvl_B
	-- end,
	
}
do
	for k,_ in pairs(sortMethods) do
		sortByList[k] = L[k]
	end
end

local function GetSortMethod()
	return sortMethods[currentSortMethod or AddonParent.db.profile.displaySortMethod]
end

-- function Module:GetSortMethod()
	-- return GetSortMethod()
-- end

function Module:SetSortMethod(method)
	if sortByList[method] then
		currentSortMethod = method
		return true
	end
	return false
end

function Module:Update_TreeView_Lists()
	TreeView_Lists = deltable(TreeView_Lists)
	TreeView_Lists = newtable()
	D("Update Tree View Lists() ")
	if type(CacheDB) ~= "table" then error("CacheDB not table on Init") end
	for trade,data in pairs(AddonParent.Cache) do			--Pairs though the cache
		D("UTVL: itterate thuogh "..trade)
		local mT = newtable()					--New Table Per-TradeSkill
		for id, _ in pairs(data) do				--pairs over the ID's
			--D("UTVL ID: "..id)
			local t = newtable()				--each craftable get's a table
			if GetLocale() == "enUS" and trade == "Enchanting" then		--Enchanting is HOMO, and dosn't do this in other locales...
				t.text = (GetSpellInfo(id)):gsub("Enchant ","")
				--t.name = (GetSpellInfo(id)):gsub("Enchant ","")
				--D("UTVL text = "..t.text)
			else
				t.text = (GetSpellInfo(id))
				--t.name = (GetSpellInfo(id))
				--D("UTVL text = "..t.text)
			end
			t.value = id						--give it a pass value
			LibGratuity:SetHyperlink("enchant:"..id)
			--D("Add "..(t.text).." to "..trade)
			tinsert(mT, t)						--add craftable to tradeSkill table
		end
		--D("UTVL: Sorting table")
		--table.sort(mT, sortMethods.ReqLvl)
		table.sort(mT, GetSortMethod())			--sort the table accordingly
		TreeView_Lists[trade] =  mT				--add to master table
		D("Added "..trade.." to Browser Cache")
	end
end
local nub = 0
function Module:Update_DropDown_List()
	--DropDown_List = deltable(DropDown_List)
	D("Update Drop Down List()" )
	DropDown_List = DropDown_List or newtable()
	local lastNub = nub
	for trade, _ in pairs(DBO:GetGuildTradesList()) do
		if not DropDown_List[trade] then
			DropDown_List[trade] = BTS[trade] or trade
			nub = nub + 1
			D("Add "..trade.." / "..BTS[trade].." to Dropdown List")
		end
	end
	if lastNub ~= nub then
		return true
	end
end

local function OpenHyperlink( owner, link )
	ShowUIPanel(GuildCraftTT);
	if ( not GuildCraftTT:IsShown() ) then
		GuildCraftTT:SetOwner(owner, "ANCHOR_NONE")
		GuildCraftTT:SetPoint("TOPLEFT", owner ,"TOPRIGHT", 20,-70)
	end
	GuildCraftTT:SetHyperlink(link);
end

local function SendHyperlinkToChat( link )
	if ChatFrameEditBox:IsVisible() then
			ChatFrameEditBox:Insert("\124cffffd000\124Henchant:"..link.."\124h["..(BTS[currentTradeSkill] or currentTradeSkill)..": "..(GetSpellInfo(link)).."]\124h\124r")
	else
		DEFAULT_CHAT_FRAME:AddMessage("\124cffffd000\124Henchant:"..link.."\124h["..(BTS[currentTradeSkill] or currentTradeSkill)..": "..(GetSpellInfo(link)).."]\124h\124r" )
	end
end

function Module:InitValues()
	--D("InitValues - CurrentTrade:"..(currentTradeSkill and currentTradeSkill or "None"))
	if self:Update_DropDown_List() then
		self:Update_TreeView_Lists()
	end
	--D("InitValues - CurrentTrade: none")
	currentTradeSkill = DropDown_List[currentTradeSkill] and currentTradeSkill or next(DropDown_List)
	if not currentTradeSkill and not next(DropDown_List) then
		--D("InitValues - CurrentTrade: still none")
		return false
	end
	--D("InitValues - CurrentTrade:"..currentTradeSkill)
	return true
end

---Ok..... Make the code look pritty and easy to read....
--This is the AceGUI Code for the browser...

do
	--Locals for all objects as we use them all over the place...
		--status
	local isBrowserShowing
		--Objects
	local browserFrame, tradeSkill_List, treeView_TradeSkills, makersLable, searchBox, sortMethodsDropDown
		--Functions
	local browserFrame_OnClose, tradeSkill_List_OnValueChange, treeView_OnSelect, treeView_OnClick
	local searchBox_OnTextChanged, sortMethodsDropDown_OnValueChange
	
	--Open-Ze-Bitch()
	
	function Module:CreateBrowser()
		if browserFrame then
			return
		else
		--Module:Update_TreeView_Lists()
		--Module:Update_DropDown_List()
		--Setup Frame , and callbacks....
		browserFrame = AceGUI:Create("Frame")
		browserFrame:SetTitle(L["Guild Craft - Browser"])
		browserFrame:SetLayout("Flow")
		browserFrame:SetCallback("OnClose", browserFrame_OnClose )
		
		--Object1: TradeSkill Drop down box --- this drop down has all the available guild craftables in it..
		tradeSkill_List = AceGUI:Create("Dropdown")
		tradeSkill_List:SetList(DropDown_List)
		tradeSkill_List:SetCallback("OnValueChanged", tradeSkill_List_OnValueChange)

		--Object2: TreeView_TradeSkills ---This is a Tree View Object, tree list shows the craftable items for the selected TradeSkill chosen via "tradeSkill_List"
		treeView_TradeSkills = AceGUI:Create("TreeGroup")
		treeView_TradeSkills.width = "fill"
		treeView_TradeSkills.height = 'fill'
		treeView_TradeSkills:SetLayout("Flow")
		treeView_TradeSkills:SetCallback("OnGroupSelected", treeView_OnSelect)
		treeView_TradeSkills:SetCallback("OnClick", treeView_OnClick)
		treeView_TradeSkills:SetTreeWidth(263, true)

			--Object 2.5: Lable Added to right hand view as a child, this displays who can craft the item, now in Alpha sorted values... yay for tables >.>
			makersLable = AceGUI:Create("Label")
			makersLable:SetText(L["Select a Trade Skill from the List above and the craftables from the left"])
			makersLable.width = "fill"
		
		--Object 3: Search Box
		searchBox = AceGUI:Create("EditBox")
		searchBox:SetLabel(L["Search"])
		searchBox:SetCallback("OnEnterPressed", searchBox_OnTextChanged)
		
		--Object 4: Sort Method Drop down box --- this drop down has all the available sort methods.
		sortMethodsDropDown = AceGUI:Create("Dropdown")
		sortMethodsDropDown:SetList(sortByList)
		sortMethodsDropDown:SetCallback("OnValueChanged", sortMethodsDropDown_OnValueChange)
		
		--Make Babies
		browserFrame:AddChild(tradeSkill_List)
		browserFrame:AddChild(searchBox)
		browserFrame:AddChild(sortMethodsDropDown)
		browserFrame:AddChild(treeView_TradeSkills)
		treeView_TradeSkills:AddChild(makersLable)
		browserFrame:Hide()
		end --end of existance statement
	end		--End of toggle Function
	
	--Functions
	function browserFrame_OnClose(widget, event, ...)
		HideUIPanel( GuildCraftTT )
		browserFrame:Hide()
		isBrowserShowing = false
		D("Close Browser from browser button or esc")
	end
	
	function tradeSkill_List_OnValueChange(widget, event, value)
		D("tradeSkill_List_OnValueChange(): "..value)
		treeView_TradeSkills:SetTree(TreeView_Lists[value])
		currentTradeSkill = value
	end
	
	function treeView_OnSelect(widget, event, value)
		D("treeView_OnSelect(): "..value)
		makersLable:SetText( L["Can be crafted by:\n"]..DBO:GetPrittyMakers(currentTradeSkill, value) )
		OpenHyperlink( treeView_TradeSkills.dragger , "enchant:"..value)
	end
	
	function treeView_OnClick(widget, event, value)
		D("treeView_OnClick(): "..value)
		if IsShiftKeyDown() then
			SendHyperlinkToChat(value)		--Add Shift here, as it's not done in SHLTC()
		end
	end
	
	local searchFuncForGratuity = function(a) if a:lower() ~= a:upper() then return "["..a:lower()..a:upper().."]" end end
	
	function searchBox_OnTextChanged(widget, event, value)
		D("searchBox_OnTextChanged(): "..value)
		value = value:lower()
		TreeView_Lists[currentTradeSkill] = deltable(TreeView_Lists[currentTradeSkill])
		TreeView_Lists[currentTradeSkill] = newtable()
		
		for id, name in pairs( DBO:GetGuildTrade(currentTradeSkill) ) do
			LibGratuity:SetHyperlink("enchant:"..id)
			if name:lower():match(value) or LibGratuity:Find( ( (value:gsub("%a", searchFuncForGratuity ) ) ) ) then
				local t = newtable()
				if GetLocale() == "enUS" and currentTradeSkill == "Enchanting" then		--Enchanting is HOMO, and dosn't do this in other locales...
					t.text = (GetSpellInfo(id)):gsub("Enchant ","")
				else
					t.text = (GetSpellInfo(id))
				end
				t.value = id						--give it a pass value
				tinsert( TreeView_Lists[currentTradeSkill],t )
			end
		end
		table.sort(TreeView_Lists[currentTradeSkill], GetSortMethod())
		
		treeView_TradeSkills:SetTree(TreeView_Lists[currentTradeSkill])
	end
	
	function sortMethodsDropDown_OnValueChange(widget, event, value)
		D("sortMethodsDropDown_OnValueChange(): "..value)
		if Module:SetSortMethod(value) then
			for craft, data in pairs(TreeView_Lists) do
				table.sort(data, GetSortMethod())
			end
			treeView_TradeSkills:SetTree(TreeView_Lists[currentTradeSkill])
		end
	end
	function Module:ToggleBrowser()
		D("Toggle Called")
		--Init Inital Values:
		if not Module:InitValues() then
			--D("InitValues Failed")
			browserFrame:Hide()
			isBrowserShowing = false
			AddonParent:Print(L["No Trades from Guild available"])
			return
		end
		--D("IntValues Suceeded, setting options")
		tradeSkill_List:SetList(DropDown_List)
		tradeSkill_List:SetValue(currentTradeSkill)
		sortMethodsDropDown:SetValue(currentSortMethod)
		treeView_TradeSkills:SetTree(TreeView_Lists[currentTradeSkill])

		if isBrowserShowing then
			D("Browser Open, Closing it")
			isBrowserShowing = false
			browserFrame:Hide()
		elseif not isBrowserShowing then
			D("Browser Not open, Showing It")
			isBrowserShowing = true
			browserFrame:Show()
		end
	end
	--Hook the escape key here....
	
	local old_CloseSpecialWindows
	
	if not old_CloseSpecialWindows then
		old_CloseSpecialWindows = CloseSpecialWindows
		CloseSpecialWindows = function()
			local found = old_CloseSpecialWindows()
			if browserFrame.frame:IsVisible() then
				D("Close Browser from Esc")
				browserFrame:Hide()
				return true
			end
			return found
		end
	end
	
end
