local rev = tonumber(("$Revision: 78723 $"):match("(%d+)")) or 0
if rev > GuildCraft.version then GuildCraft.version = rev end

local function D(msg)
	if (not GuildCraft ) or (not GuildCraft.db.global.debug.outofguild) then return end
	ChatFrame1:AddMessage("|cff33FF66Guild Craft(OOG):|r "..msg)
end

local AddonParent = LibStub("AceAddon-3.0"):GetAddon("GuildCraft")
local module = AddonParent:NewModule("Non-Guild-Comms", "AceComm-3.0", "AceSerializer-3.0", "AceEvent-3.0") --Get Toys  "AceConsole-3.0"
module.rev = rev
local self_player = UnitName("player")
local Addon_Prefix = "GuildCraft_OOG"
local L = LibStub("AceLocale-3.0"):GetLocale("GuildCraft")
local DBO
local dbRemoteInfo, parentDB

function module:OnInitialize()
	parentDB = AddonParent.db
	module:SetEnabledState( parentDB.profile.moduleStatus.OutOfGuild )
	parentDB.RegisterCallback(self, "OnProfileChanged","ProfileChanged")
	parentDB.RegisterCallback(self, "OnProfileReset","ProfileChanged")
	parentDB.RegisterCallback(self, "OnDatabaseReset","ProfileChanged")
	D("Out of Guild -- On Init")
end

function module:OnEnable()
	dbRemoteInfo = AddonParent.db.factionrealm.RemoteInfo
	--dbCache = AddonParent.db.factionrealm.Cache
	self:RegisterComm(Addon_Prefix)
	DBO = AddonParent:GetModule("DBO")
	AddonParent:Print(L["Out of Guild Communications Module Enabled"])
end

function module:OnDisable()
	AddonParent:Print(L["Out of Guild Communications Module |cffff3300Disabled|r"])
	self:UnregisterComm(Addon_Prefix)
end

---handle Profile changes here...

function module:ProfileChanged(event, db, key)
	if db.profile.moduleStatus.OutOfGuild then
		self:Enable()
	else
		self:Disable()
	end
end

local function DispatchComms(sender, ...)
	local command = (select(2,...))
	if module[command] then
		D("Dispatching Comms from: "..sender.." '"..command.."'")
		module[command](module, sender, select(3,...))
	end
end

function module:OnCommReceived(prefix, message, distribution, sender)
	if prefix ~= Addon_Prefix then return end
	DispatchComms(sender, self:Deserialize(message))
end

local SAM_Whisper, GetLocalVersion, GetRemoteVersion
do
	function Cerial(...)
		return module:Serialize(...)
	end

	function SAM_Whisper(player, ...)
		D("Send Whisper to "..player.." '"..( tostring( select(1,...) ) ) )
		module:SendCommMessage(Addon_Prefix, Cerial(...), "WHISPER", player, "BULK")
	end

	function GetLocalVersion(sender)
		return AddonParent.db.char.dbVersion
	end

	function GetRemoteVersion(sender)
			return dbRemoteInfo[sender] and dbRemoteInfo[sender].dbVersion
	end
end

function module:RequestCurrentVersion( player )
	D("RequestCurrentVersion from "..player)
	--if player then
		SAM_Whisper(player , "VersionUpdateRequest")
	--end
end

function module:VersionUpdateRequest( sender )
	D("Version Update Request() from: "..sender)
	SAM_Whisper( sender, "HostedVersion", GetLocalVersion()  )
end

function module:HostedVersion(sender, version)
	version = tonumber(version)
	local rVer = tonumber(GetRemoteVersion(sender) or "-1")
	if (rVer and rVer < version) or (not rVer) then
		D("Hosted Version() from "..sender)
		SAM_Whisper( sender, "GiveHostedVerion" )
	end
end

function module:GiveHostedVerion( sender )
	D("Give Hosted Version to "..sender)
	SAM_Whisper( sender, "UpdateTradeInfo", GetLocalVersion(), AddonParent.db.char.Share )
end

function module:UpdateTradeInfo(sender, version, tbl)
	version = tonumber(version)
	D("Update Trade Info() from: "..sender.." ver: "..version)
	local rVer = tonumber(GetRemoteVersion(sender) or "-1")
	if (rVer and rVer < version) or (not rVer) then
		DBO:UpdateRemoteSender(sender,version, tbl)
		AddonParent:Print(L["Updated %s's tradeskills. Version: %s"], sender, version)
	end
end

local function ToggleModuleState( state )
	if not parentDB.profile.moduleStatus.OutOfGuild then
		module:Enable()
		parentDB.profile.moduleStatus.OutOfGuild = true
	else
		module:Disable()
		parentDB.profile.moduleStatus.OutOfGuild = false
	end
end

AddonParent.optionsTable.args.OutOfGuild = {
	name = L['GuildCrafts - Out of Guild Config'],
	handler = module,
	type = 'group',
	args = {
		state = {
			type = 'toggle',
			name = L["Enable out of Guild Synch"],
			order = 10,
			get = function(info)
					return module:IsEnabled()
				end,
			set = function(info, vlaue)
					return ToggleModuleState()
				end,
		},
		AddPlayeInfor = {
			type = 'description',
			name = L["Desc_Info_for_OOG"],
			order = 20,
			},
		SendToPlayer = {
			type = 'input',
			name = L["Send To Player"],
			order = 30,
			disabled = function() return not module:IsEnabled() end,
			set = function(info, value)
					parentDB.profile.quickListOutOfGuild[value] = value
					module:GiveHostedVerion( value )
					AddonParent:Print(L["Craftables Sent To: %s"], value)
				end,
		},
		SendToRecentPlayer = {
			type = 'select',
			name = L["Send to Recent Player"],
			values = function() return parentDB.profile.quickListOutOfGuild end,
			style = "dropdown",
			order = 40,
			disabled = function() return not module:IsEnabled() end,
			set = function(info, value)
					module:GiveHostedVerion( value )
					AddonParent:Print(L["Craftables Sent To: %s"], value)
				end,
		},
		brake = {
			type = 'description',
			name = '',
			order = 50,
		},
		RequestFromPlayer = {
			type = 'input',
			name = L["Request From Player"],
			order = 60,
			disabled = function() return not module:IsEnabled() end,
			set = function(info, value)
					parentDB.profile.quickListOutOfGuild[value] = value
					module:RequestCurrentVersion( value )
				end,
		},
		ReqRecentPlayer = {
			type = 'select',
			name = L["Request from Recent Player"],
			values = function() return parentDB.profile.quickListOutOfGuild end,
			style = "dropdown",
			order = 70,
			disabled = function() return not module:IsEnabled() end,
			set = function(info, value)
					module:RequestCurrentVersion( value )
				end,
		}
	}
}