--ChoicesUI
local rev = tonumber(("$Revision: 78723 $"):match("(%d+)")) or 0
if rev > GuildCraft.version then GuildCraft.version = rev end

--Debug and table functions...
local D, newTable, delTable, clearTable
do
	function D(msg)
		if (not GuildCraft ) or (not GuildCraft.db.global.debug.tradeskill) then return end
		DEFAULT_CHAT_FRAME:AddMessage("|cff33FF66GC2-TSS:|r "..msg)
	end

-- Table Pool for recycling tables
	local tablePool = {}
	setmetatable(tablePool, {__mode = "kv"});	-- Weak table

	function newTable()
		local t = next(tablePool) or {};
		tablePool[t] = nil;
		return t;
	end

	function delTable(t)
		if (type(t) == "table") then
			for k, v in pairs(t) do
				if (type(v) == "table") then
					delTable(v);	-- child tables get put into the pool
				end
				t[k] = nil;
			end
			setmetatable(t, nil);
			t[true] = true;
			t[true] = nil;
			tablePool[t] = true;
		end
		return nil;
	end

	function clearTable(t, recycleSubTables)
		for k,v in pairs(t) do
			if recycleSubTables and type(v) == 'table' then
				t[k] = delTable(t[k])
			else
				t[k] = nil
			end
		end
		return t
	end
end

local addonParent = LibStub("AceAddon-3.0"):GetAddon("GuildCraft")
local module = addonParent:NewModule("CraftSelect", "AceEvent-3.0", "AceConsole-3.0")
module.rev = rev
local BTS = LibStub("GuildCraft_BabbleTradeSkill_Mini")
local L = LibStub("AceLocale-3.0"):GetLocale("GuildCraft", true)
local LibGratuity = LibStub("LibGratuity-3.0")
local dbTradeSkills

local optionsTable = {
		type = "group",
		name = L["Guild Craft Share Panel"],
		handler = module,
		childGroups = "tab",
		args = {
			sendToGuild = { type = 'execute', name = L["Publish To Guild"], order = 1,
				func = function()
					module:UpdateVersionHash()
					module:RefreshShareTable()
					if addonParent:GetModule("CommsIO",true) then
						addonParent:GetModule("DBO"):PublishToGuild()
					end
				end,
			},
			resetDB = { type = 'execute', name = L["Reset Entire Database"] , order = 2, confirm = true,
				func = function()
						addonParent.db:ResetDB()
						D(L["Database Reset"])
					end,
				hidden = function(info) return not addonParent.db.global.MasterFlag end,
				confirmText = L["This Resets All parts of GuildCrafts, from the Cache, to the Guild Info, to local Trade Skill data FOR ALL CHARATURES EVERYWHERE. You'll Need to Relog to update from the Guild"],
			},
			version = {type = 'description', order = 2.5, name = function() return L["DB Version: "]..addonParent.db.char.dbVersion end, },
			help = { type = 'description', order = 3,
				name = L["Here select what items you'd like to share to the Guild and Others. \nIf your not seeing anything here, try opening up a Trade Skill window."],
			},
		},
	}

local dbTradeSkills, dbShare

function module:OnEnable()
	dbTradeSkills = addonParent.db.char.TradeSkills
	dbShare = addonParent.db.char.Share
	addonParent.optionsTable.args.ChoiceUI = optionsTable
	self:RegisterEvent("TRADE_SKILL_SHOW")
	self:RegisterEvent("CRAFT_SHOW")
	self:InitOptionsTable()
	D("TradeSkillScanner & UI Loaded")

end

function module:AddLocalTradeInfo(trade, link)
	if not dbTradeSkills[trade] then
		dbTradeSkills[trade] = newTable()
	end
	dbTradeSkills[trade][link] = dbTradeSkills[trade][link] or false
end

function module:GetBindOnPickupFlag(link)
	link = tonumber(link)
	LibGratuity:SetHyperlink("enchant:"..link)
	return LibGratuity:Find(ITEM_BIND_ON_PICKUP) and true or false
end

function module:UpdateVersionHash()
	addonParent.db.char.dbVersion = date("!%Y%m%d%H%M%S")
end

function module:RefreshShareTable()
	dbShare = clearTable(dbShare, true)
	for name, data in pairs(dbTradeSkills) do
		D("RFST: trade: "..name.." Data Type: "..type(data))
		dbShare[name] = newTable()
		for k, v in pairs(data) do
			D("RFST Dive: k = "..k.." v = "..(v and "true" or "false"))
			if v then
				D("RFST: trade: "..name.." item: "..(v and "true" or "false"))
				dbShare[name][k] = v
			end
		end
		if (not next(dbTradeSkills[name])) then
			dbTradeSkills[name] = delTable(dbTradeSkills[name])
		end
	end
	D( (dbShare == addonParent.db.char.Share and "db Share true" or "dbShare not true") )
end

function module:InitOptionsTable()
	local argTbl = optionsTable.args
		for trade, tData in pairs(dbTradeSkills) do
			if type(tData) == 'table' then
				argTbl[trade] = argTbl[trade] or { type = 'group', name = BTS[trade],
					args = {
						select_All = { type = 'execute', name = L['Select All'], order = 1, func = "SelectAllTrade", },
						select_None = { type = 'execute', name = L['Unselect All'], order = 2, func = "SelectNoneTrade",},
						spacer = {type = 'description', name = "", order = 3},
						}
					}
				for id , _ in pairs(tData) do
					--D("Link: "..id)
					if not argTbl[trade].args[id] then
						local realName = (GetSpellInfo(id)):gsub( BTS[trade]..":","") --:gsub(L["^Enchant%s"],"")
						argTbl[trade].args[id] = { type = 'toggle', name = realName,
							desc = L["Enable or Disable sharing %s to guild"]:format(realName),
							get = "GetTradeStatus", set = "SetTradeStatus", }
				end
			end
		end
	end
end

function module:GetTradeStatus(info)
	--D("GTS link: "..info[#info])
	--D("GTS trade: "..info[#info-1])
	return dbTradeSkills[ info[#info-1] ][ info[#info] ]
end

function module:SetTradeStatus(info, val)
	--D("STS link: "..info[#info])
	--D("STS trade: "..info[#info-1])
		if val then
			--D("STS val: True")
		else
			--D("STS val: False")
		end
	dbTradeSkills[ info[#info-1] ][ info[#info] ] = val
end

function module:SelectAllTrade(info)
	--D("SAT trade: "..info[#info-1])
	local tradeTable = dbTradeSkills[ info[#info-1] ]
	for k,_ in pairs(tradeTable) do
		if not module:GetBindOnPickupFlag(k) then
			--D("SAT-trade: "..info[#info-1].." item: "..k.." set true")
			tradeTable[k] = true
		end
	end
end

function module:SelectNoneTrade(info)
	--D("USAT trade: "..info[#info-1])
	local tradeTable = dbTradeSkills[ info[#info-1] ]
	for k,_ in pairs(tradeTable) do
		--D("USAT-trade: "..info[#info-1].." item: "..k.." set false")
		tradeTable[k] = false
	end
end
--[[----------------------------------------------
TradeSkill Scanner
------------------------------------------------]]
	--[[
	Ordering
		1 = select all
		2 = select none
		3 = spacer
		4 = Orange craft	|cffff6600
		5 = Yellow craft	|cffffff33
		6 = Green Craft	|cff66ff33
		7 = Gray Craft	|cffcccccc

		skill_order
		skill_color
		color_end
	]]--

local IgnoreList = {
	["First Aid"] = true,
	["Poisons"] = true,
	["Mining"] = true,
	["Beast Training"] = true,
}

local skill_color = {
	["trivial"] = "|cffcccccc",
	["easy"] = "|cff66ff33",
	["medium"] = "|cffffff33",
	["optimal"] = "|cffff6600",
}

local skill_order = {
	["trivial"] = 7,
	["easy"] = 6,
	["medium"] = 5,
	["optimal"] = 4,
}
local color_end = "|r"

local enchantLink, name, status, _

function module:TRADE_SKILL_SHOW()
	local craftName = BTS:ReverseTranslate((GetTradeSkillLine()))
	D("T_S_S Event "..(craftName and craftName or "").." craft Found")
	if not craftName then return D( GetTradeSkillLine() or "??".." Not a Crafting Skill.") end
	if IgnoreList[craftName] then return D("tradeskill: "..craftName.." is being Ignored.") end
	--Checks
	if not ( GetNumTradeSkills() > 0) then return end
	local argTbl = optionsTable.args
	argTbl[craftName] = argTbl[craftName] or {
				type = 'group',
				name = GetTradeSkillLine(),
				args = {
						select_All = {
							type = 'execute',
							name = L['Select All'],
							order = 1,
							func = "SelectAllTrade",
						},
						select_None = {
							type = 'execute',
							name = L['Unselect All'],
							order = 2,
							func = "SelectNoneTrade",
						},
						spacer = {type = 'description', name = "", order = 3},
					}
				}
	D("Itterate though Craftables")
	for i = 1, GetNumTradeSkills() do
		local link = GetTradeSkillRecipeLink(i)
		if link then
			D("Found Link "..link)
			enchantLink = string.match( link , "^\124%x+\124Henchant:(.+)\124h%[(.+)%]")
			name, status = GetTradeSkillInfo(i)
			self:AddLocalTradeInfo( craftName, enchantLink)
			name = skill_color[status]..name..color_end
			D("Formatted Name "..name)
			argTbl[craftName].args[enchantLink] = {
						type = 'toggle',
						name = name,
						order = skill_order[status],
						desc = L["Enable or Disable sharing %s to guild"]:format(name),
						get = "GetTradeStatus", set = "SetTradeStatus",
					}
		end
	end
end

function module:CRAFT_SHOW()
	local craftName = BTS:ReverseTranslate(GetCraftName())
	D("C_S_S Event "..(craftName and craftName or "").." craft Found")
	if not craftName then return D( GetTradeSkillLine() or "??".." Not a Crafting Skill.") end
	if IgnoreList[craftName] then return D("Craft: "..craftName.." is being Ignored.") end
	local argTbl = optionsTable.args
	argTbl[craftName] = argTbl[craftName] or {
				type = 'group',
				name = GetCraftName(),
				args = {
						select_All = {
							type = 'execute',
							name = L['Select All'],
							order = 1,
							func = "SelectAllTrade",
						},
						select_None = {
							type = 'execute',
							name = L['Unselect All'],
							order = 2,
							func = "SelectNoneTrade",
						},
						spacer = {type = 'description', name = "", order = 3},
					}
				}
	D("Itterate though Craftables")
	for i = 1, GetNumCrafts() do
		local link = GetCraftRecipeLink(i)
		if link then
			D("Found Link "..link)
			enchantLink = string.match( link , "^\124%x+\124Henchant:(.+)\124h%[(.+)%]")
			name,_, status = GetCraftInfo(i)
			self:AddLocalTradeInfo( craftName, enchantLink)
			name = name:gsub("Enchant ", "")		--may or may not still need this, prolly can leave unlocalized as only enUS adds fking titles
			name = skill_color[status]..name..color_end
			D("Formatted Name "..name)
			argTbl[craftName].args[enchantLink] = {
						type = 'toggle',
						name = name,
						order = skill_order[status],
						desc = L["Enable or Disable sharing %s to guild"]:format(name),
						get = "GetTradeStatus", set = "SetTradeStatus",
					}
		end
	end
end

function module:GetShareTable()
	return dbShare
end

