--Core.lua
--$Date: 2008-08-15 13:21:10 -0400 (Fri, 15 Aug 2008) $ / $Revision: 80492 $
--[[
Copyright (c) 2007 by Orionshock

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

All files included in this archive other than he included librarys, an assortment of Ace3Core Librarys and LibGratuity,  are licensed  under the above.
All other files are licensed under their respective licenses.

]]--

--local lastAddonDBCompatVersion = 5
local lastAddonDBCompatVersion = 7500	--WowaceDistro Option

--Debug and table functions...
local D, newTable, delTable, clearTable
do
	function D(msg,...)
		if (not GuildCraft ) or (not GuildCraft.db.global.debug.core) then return end
		if (...) then
			local info = ""
			for i = 1, select("#",...) do
				local var = select(i, ...)
				if type(var) == "table" then
					var = ", table"..tostring(var).." "
				elseif type(var) == "boolean" then
					var = var and ", true" or ", false"
				elseif type(var) == "nil" then
					var = ", nil"
				else
					var = " "..var
				end
				info = info..var
			end
			DEFAULT_CHAT_FRAME:AddMessage("|cff33FF66GC-Core:|r "..msg..info )
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cff33FF66GC2-Core:|r "..msg)
		end
	end

-- Table Pool for recycling tables
	local tablePool = {}
	setmetatable(tablePool, {__mode = "kv"});	-- Weak table

	function newTable()
		local t = next(tablePool) or {};
		tablePool[t] = nil;
		return t;
	end

	function delTable(t)
		if (type(t) == "table") then
			for k, v in pairs(t) do
				if (type(v) == "table") then
					delTable(v);	-- child tables get put into the pool
				end
				t[k] = nil;
			end
			setmetatable(t, nil);
			t[true] = true;
			t[true] = nil;
			tablePool[t] = true;
		end
		return nil;
	end

	function clearTable(t, keepRoot)
		for k,v in pairs(t) do
			if keepRoot and type(v) == 'table' then
				t[k] = delTable(t[k])
			else
				t[k] = nil
			end
		end
		return t
	end
end


---DB Structure..
local masterDB_Defaults = {
	char = {
		TradeSkills = {
			-- ["TradeSkill"] = {
				-- ["ItemID"] = false
				-- ["ItemID"] = false
				-- ["ItemID"] = false
			-- },
			-- ["TradeSkill"] = {
				-- ["ItemID"] = false
				-- ["ItemID"] = false
				-- ["ItemID"] = false
			-- },
		},
		dbVersion = -1,
		Share = {
			-- ["TradeSkill"] = {
				-- ["ID"] = true,
				-- ["ID"] = true,
				-- ["ID"] = true,
			-- },
			-- ["TradeSkill"] = {
				-- ["ItemName"] = Boolen_value_choice,
				-- ["ItemName"] = Boolen_value_choice,
				-- ["ItemName"] = Boolen_value_choice,
			-- },
		},
	},	--End Char DBO
	profile = {
		moduleStatus = {
			OutOfGuild = false,
			},
		displaySortMethod = "Name",
		quickListOutOfGuild = {
			},
		},
	factionrealm = {
		--dbVersion == metatable setup by OfflineSync module
		RemoteInfo = {
			-- ["GuildMember"] = {
				-- ["dbVersion"] = Date_Time_Stamp_UTC,
				--["trade"] = {
					-- ["ID"] = true,
					-- ["ID"] = true,
					-- ["ID"] = true,
				--},
			-- },
			-- ["GuildMember"] = {
				-- ["dbVersion"] = Date_Time_Stamp_UTC,
				--["trade"] = {
					-- ["ID"] = true,
					-- ["ID"] = true,
					-- ["ID"] = true,
				--},
			-- },
			-- ["GuildMember"] = {
				-- ["dbVersion"] = Date_Time_Stamp_UTC,
				--["trade"] = {
					-- ["ID"] = true,
					-- ["ID"] = true,
					-- ["ID"] = true,
				--},
			-- },
		}, -- End of RemoteInfo
		guildSettings = {
			[""] = {
				locked = false,
				blacklisted = {},
				}
		},
	}, -- End of faction table
	global = {
		debug = {
			core = false,
			tradeskill = false,
			dbo = false,
			comms = false,
			browser = false,
			outofguild = false,
			fubar = false,
		},
		playerVersions = {
		},
	},
}

--[[
Core:
 ]]--

local dbTradeSkills, dbShare
GuildCraft = LibStub("AceAddon-3.0"):NewAddon("GuildCraft", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
GuildCraft.version = (tonumber(("$Revision: 80492 $"):match("(%d+)")) or 0 )
GuildCraft.version = GuildCraft.version
local core = GuildCraft
local L = LibStub("AceLocale-3.0"):GetLocale("GuildCraft")
local player_name, guildSettings
core.guildRight = {}
core.guildMembers = {}
core.offline = {}

core.optionsTable = {
	type = "group",
	name = L["Guild Craft"],
	handler = core,
	childGroups = "tab",
	args = {
		special = { type = "group", name = "Special Stuff", order = -1,
			hidden = function(info)
							local rank = select(3, GetGuildInfo("player"))
								-- if GuildCraft.db.global.MasterFlag then
									-- rank = false
								-- end
							return (not (rank<=1))
						end,
			args = {
				lockDB = {
					name = "Lock Guild Wide DB", type = "toggle", order = 1,
					get = function()
							return guildSettings.locked
						end,
					set = function(info, value)
							guildSettings.locked = value
							core:ChangeGMOpt("ToggleDBOLocked" , value)
						end,
					},
				ClearGuildDB = {
					name = "Clear Guild Wide Database", type = "execute", order = 2,
					func = function(info)
						core:ChangeGMOpt("ClearGuildDB")
						end,
					confirm = function() return "This will reset the database for everyone ONLINE! They will have to resetup the addon" end
					},
				blacklist = {
					name = "Blacklist Guild Member", type = "multiselect", order = 5,
					get = function(info, get)
							if guildSettings.blacklisted[get] then
								return true
							else
								return false
							end
						end,
					set = function(info, key, value)
						if value and core.guildMembers[key] and (not core.guildRight[key]) then
							core:ChangeGMOpt("AddBlacklist", key, date("!%Y%m%d%H%M%S"))
								guildSettings.blacklisted[key] = true
						elseif not value then
							core:ChangeGMOpt("RemoveBlackList", key, date("!%Y%m%d%H%M%S"))
								guildSettings.blacklisted[key] = nil
						end
					end,
					values = function(info)
						return core.guildMembers
					end,
					},
				},
			},
				enableDebug = { name = (L["Enable Debug"]), type = "group", order = -10,
			args = {
			AddPlayeInfor = { name = L["Debug_Desc"], type = 'description', order = 1,},
				core = { name = (L["Core"]), type = "toggle", order = 3,
					get = function() return core.db.global.debug.core end,
					set = function(info, status) core.db.global.debug.core = status end,},
				tradeskill = { name = (L["Tradeskill Module"]), type = "toggle", order = 4,
					get = function() return core.db.global.debug.tradeskill end,
					set = function(info, status) core.db.global.debug.tradeskill = status end,},
				dbo = { name = (L["Database Module"]), type = "toggle", order = 5,
					get = function() return core.db.global.debug.dbo end,
					set = function(info, status) core.db.global.debug.dbo = status end,},
				comms = { name = (L["Communications Module"]), type = "toggle", order = 6,
					get = function() return core.db.global.debug.comms end,
					set = function(info, status) core.db.global.debug.comms = status end,},
				browser = { name = (L["Browser UI Module"]), type = "toggle", order = 7,
					get = function() return core.db.global.debug.browser end,
					set = function(info, status) core.db.global.debug.browser = status end,},
				outofguild = { name = (L["Out of Guild Module"]), type = "toggle", order = 8,
					get = function() return core.db.global.debug.outofguild end,
					set = function(info, status) core.db.global.debug.outofguild = status end,},
				fubar = { name = (L["FuBar Module"]), type = "toggle", order = 9,
					get = function() return core.db.global.debug.fubar end,
					set = function(info, status) core.db.global.debug.fubar = status end,},
				all = { name = (L["Enable All"]), type = "execute", order = 10,
					func = function(info)
							for k, v in pairs(core.db.global.debug) do
								core.db.global.debug[k] = true
							end
						end},
				none = { name = (L["Disable All"]), type = "execute", order = 11,
					func = function(info)
							for k, v in pairs(core.db.global.debug) do
								core.db.global.debug[k] = false
							end
						end},
				},
			},
		},
	}

function core:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("GuildCraftMasterDB" , masterDB_Defaults )
	if (not self.db.global.AddonDBVersion) or (self.db.global.AddonDBVersion < lastAddonDBCompatVersion) then
		self.db:ResetDB()
		D(L["Database reset, please resetup the addon"])
	end
	self.db.global.AddonDBVersion = self.version
	self.Cache = {}
end

local timmer
function core:OnEnable()
	player_name = UnitName("player")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Guild Craft", self.optionsTable)
	self:RegisterChatCommand("guildcraft", function()
		LibStub("AceConfigDialog-3.0"):Open("Guild Craft")
	end )
	--self.version = self.version.."-Curse-Distro"
	self.version = self.version.."-WoWAce-Distro"
	self:Print(L["Loaded, /guildcraft to bring up config, /gcb for the browser. Version: %s"], GuildCraft.version)
	self:RegisterEvent("GUILD_ROSTER_UPDATE")
	GuildRoster()
	self:GetModule("CommsIO"):SoftEmbed(self)
	if not self:InitGuildManagement() then
		timmer = self:ScheduleRepeatingTimer( "InitGuildManagement", 10)
	end
	D("Guild Craft Core Loaded")
end

function core:InitGuildManagement()
	local guild = GetGuildInfo("player")
	if not guild then
		D("No Guild Info at loop")
		return
	end
	if not self.db.factionrealm.guildSettings[guild] then
		guildSettings = {
			guildName = guild,
			locked = false,
			blacklisted = {},
		}
		self.db.factionrealm.guildSettings[guild] = guildSettings
	else
		guildSettings = self.db.factionrealm.guildSettings[guild]
	end
	self:RegisterCommsCall("GuildManagement")
	self:RegisterCommsCall("GetCurrentGuildSettings")
	self:SendToGuild("GetCurrentGuildSettings")
	self:CancelTimer(timmer, true)
	return true
end

function core:OnDisable()
	self.db.factionrealm.guildSettings[GetGuildInfo("player")] = guildSettings
end

function core:GetRightToManageDB(name)
	local Right = self.guildRight[name] and true or false
		if self.db.global.MasterFlag then
			Right = true
		end
	return Right
end

local checked_offline

function core:GUILD_ROSTER_UPDATE()
	D("Guild Roster Update")
	local numGuildies = GetNumGuildMembers(true)
	D("NumberGuildies = "..(numGuildies or "ERRROR"))
	for i = 1, numGuildies do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i)
		--D("GRU: "..name..", rank = "..(rankIndex or "ERROR"))
		if rankIndex == (1 or 0) then
			--D("GRU: "..name.." is officer")
			self.guildRight[name] = true
		end
		if self.guildRight[name] and (not rankIndex == (1 or 0 ) ) then
			--D("GRU: "..name.." is no longer officer")
			self.guildRight[name] = nil
		end
		--D("GRU: "..name.." is guildie")
		self.guildMembers[name] = name
		if not online then
			self.offline[name] = true
		end
	end
	local numguildright = 0
	for k,v in pairs(self.guildRight) do
		numguildright = numguildright + 1
	end
	D("GRU done with guild itter. "..numguildright.." officers")
	if not checked_offline then
		D("Offline Quiry Check")
		if self:GetModule("DBO") then
			self:GetModule("DBO"):StarOfflineCheck()
			checked_offline = true
		end
	end
	D("GRU End")
end

function core:ChangeGMOpt(option , value, ...)
	if self.guildRight[player_name] or self.db.global.MasterFlag then
		self:SendToGuild("GuildManagement", option, value)
	end
end

function core:GuildManagement(event, sender, subCommand, ...)
	D("GuildManagement Call by: ", sender)
	if (self.guildRight[sender]) or self.db.global.MasterFlag  then
		D("GM: ", subCommand, ...)
		local var1,var2,var3 = select(1, ...)
		if subCommand == "SetGuildSettigs" then
			guildSettings.locked = var1
			guildSettings.blacklisted = var2
			self:GetModule("DBO"):SetDBOLocked(guildSettings.locked, sender)
			self:GetModule("DBO"):PurgeBlacklisted( unpack(var2)  )
		elseif subCommand == "ToggleDBOLocked" then
			guildSettings.locked = select(1, ...)
			self:GetModule("DBO"):SetDBOLocked(guildSettings.locked, sender)
		elseif subCommand == "AddBlacklist" then
			guildSettings.blacklisted[var1] = true
			self:GetModule("DBO"):PurgeBlacklisted(var1)
			self:Print(L["%s added %s to blacklist."], sender, var1)
		elseif subCommand == "RemoveBlackList" then
			guildSettings.blacklisted[var1] = nil
			self:Print(L["%s removed %s from blacklist."], sender, var1)
		elseif subCommand == "ClearGuildDB" then
			if sender == player_Name then
				self:Print("Guild Wide DB Clear Sent, your database has been cleared")
			else
				self:Print("%s has ordered that the Guild's Guild Database for everyone online be reset", sender)
			end
			-- self:ResetDB()
		end
	end
end

function core:GetCurrentGuildSettings(event, sener, ...)
	if self:GetRightToManageDB(player_Name) then
		self:SendToGuild("GuildManagement", "SetGuildSettings", guildSettings.locked, guildSettings.blacklisted)
	end
end

local prefix = "|cff33FF66"..L["Guild Craft"]..":|r "
function core:Print(match, ...)
	if string.find(match, "%%") then
		DEFAULT_CHAT_FRAME:AddMessage(prefix..string.join(", ", match:format(...) ) )
	else
		DEFAULT_CHAT_FRAME:AddMessage(prefix..string.join(", ", match, ... ) )
	end
end
